package com.hammingweight.hammockexamples.example3;

import java.io.UnsupportedEncodingException;

import jmunit.framework.cldc10.TestCase;

import com.hammingweight.hammock.Hammock;
import com.hammingweight.hammock.IArgumentMatcher;
import com.hammingweight.hammock.PopulateArrayMatcher;
import com.hammingweight.hammockexamples.mocks.MockPseudoRandomByteStream;

public class StreamCipherTest extends TestCase {

	// The OUT.
	private StreamCipher streamCipher;

	// The handler and mock(s).
	private Hammock hammock;
	private MockPseudoRandomByteStream mockStream1, mockStream2;

	public void setUp() {
		this.hammock = new Hammock();
		this.mockStream1 = new MockPseudoRandomByteStream(this.hammock);
		this.mockStream2 = new MockPseudoRandomByteStream(this.hammock);
		this.streamCipher = new StreamCipher(this.mockStream1, this.mockStream2);
	}

	public void testEncrypt() throws UnsupportedEncodingException {
		// Expectations.
		byte[] b = { 1, 2, 3, 4, 5 };
		this.hammock.setExpectation(
				MockPseudoRandomByteStream.MTHD_GENERATE_BYTES_$_ARRAY_BYTE,
				this.mockStream1, new Object[] { b }).setArgumentMatcher(0,
				new IArgumentMatcher() {

					public boolean areArgumentsEqual(Object expected,
							Object actual) {
						byte[] e = (byte[]) expected;
						byte[] a = (byte[]) actual;
						System.arraycopy(e, 0, a, 0, e.length);
						return true;
					}

				});

		// Exercise.
		byte[] msg = "hello".getBytes("UTF8");
		byte[] ciphertext = this.streamCipher.encrypt(msg);
		assertEquals(5, ciphertext.length);
		for (int i = 0; i < 5; i++) {
			assertEquals(msg[i] ^ (i + 1), ciphertext[i]);
		}

		// Verify.
		this.hammock.verify();
	}

	public void testDecrypt() {
		// Expectations.
		byte[] b = { 0x55, 0x55, 0x55, 0x55 };
		this.hammock.setExpectation(
				MockPseudoRandomByteStream.MTHD_GENERATE_BYTES_$_ARRAY_BYTE,
				new Object[] { b }).setArgumentMatcher(0,
				new PopulateArrayMatcher(0, 4, 0));

		// Exercise.
		byte[] msg = { 1, 2, 4, 8 };
		byte[] plaintext = this.streamCipher.decrypt(msg);
		assertEquals(4, plaintext.length);
		for (int i = 0; i < 4; i++) {
			assertEquals(msg[i] ^ 0x55, plaintext[i]);
		}

		// Verify.
		this.hammock.verify();
	}

	public StreamCipherTest() {
		super(2, "StreamCipherTest");
	}

	public void test(int testNum) throws Throwable {
		switch (testNum) {
		case 0:
			testEncrypt();
			break;

		case 1:
			testDecrypt();
			break;

		default:
			fail("No such test.");
		}
	}

}
