/*
 * AMethodWrapper.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

/**
 * The AMethodWrapper models a Java method. It is extended by classes that model
 * constructors and other methods.
 * 
 * @author C.A. Meijer
 */
public abstract class AMethodWrapper {

	/**
	 * The arguments associated with the method or constructor.
	 */
	private Arguments args;

	/**
	 * The exceptions thrown by the method or constructor.
	 */
	private Exceptions exceptions;

	/**
	 * The constructor.
	 * 
	 * @param args
	 *            The arguments of the method.
	 * 
	 * @param exceptions
	 *            The exceptions thrown by the method.
	 */
	public AMethodWrapper(Arguments args, Exceptions exceptions) {
		this.args = args;
		this.exceptions = exceptions;
	}

	/**
	 * Returns the arguments of the method.
	 * 
	 * @return The method's arguments.
	 */
	public final Arguments getArguments() {
		return this.args;
	}

	/**
	 * Returns the exceptions thrown by the method.
	 * 
	 * @return The exceptions thrown by the method.
	 */
	public final Exceptions getExceptions() {
		return this.exceptions;
	}
}
