package com.hammingweight.hammock;

import org.easymock.classextension.EasyMock;

import junit.framework.TestCase;

public class InvocationVerifierTest extends TestCase {

	private InvocationMatcher matcher = EasyMock
			.createMock(InvocationMatcher.class);
	private MethodInvocation mi1 = EasyMock.createMock(MethodInvocation.class);
	private MethodInvocation mi2 = EasyMock.createMock(MethodInvocation.class);
	private MethodInvocation mi3 = EasyMock.createMock(MethodInvocation.class);
	private MethodInvocation mi4 = EasyMock.createMock(MethodInvocation.class);
	private MethodInvocation mi5 = EasyMock.createMock(MethodInvocation.class);
	private MethodInvocation[] mi = { mi1, mi2, mi3, mi4, mi5 };
	private IArgumentMatcher argMatcher = EasyMock
			.createMock(IArgumentMatcher.class);

	public void testAsExpected() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testNotAsExpected() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(false);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertFalse(verifier.isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testSetInvocationCountOk() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.setInvocationCount(3).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testSetInvocationCountOk2() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(false);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testSetInvocationCountTooFew() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertFalse(verifier.setInvocationCount(3).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testSetInvocationCountTooMany() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertFalse(verifier.setInvocationCount(3).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testSetInvocationCountOk3() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.setInvocationCount(3).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testSetInvocationCountOk4() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.setInvocationCount(1, 4).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testBadConstructors() {
		try {
			new InvocationVerifier(null, this.mi);
			fail();
		} catch (NullPointerException e) {

		}

		try {
			new InvocationVerifier(this.matcher, null);
			fail();
		} catch (NullPointerException e) {

		}
	}

	public void testBadInvocationCounts() {
		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		try {
			verifier.setInvocationCount(-1);
			fail();
		} catch (IllegalArgumentException e) {

		}
		try {
			verifier.setInvocationCount(4, 3);
			fail();
		} catch (IllegalArgumentException e) {

		}
	}

	public void testSetInvocationsTwice() {
		InvocationVerifier verifier = new InvocationVerifier(
				new InvocationMatcher(new MockMethod("", 3, null, true)),
				new MethodInvocation[0]);
		verifier.setInvocationCount(3);
		try {
			verifier.setInvocationCount(3);
			fail();
		} catch (HammockException e) {

		}
	}

	public void testSetArgumentMatcher() {
		EasyMock.expect(this.matcher.setArgumentMatcher(3, argMatcher))
				.andReturn(this.matcher);
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.setArgumentMatcher(3, this.argMatcher).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testIgnoreArgument() {
		EasyMock.expect(this.matcher.ignoreArgument(3)).andReturn(this.matcher);
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertTrue(verifier.ignoreArgument(3).isVerified());

		EasyMock.verify(this.matcher);
	}

	public void testVerifyOk() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		verifier.verify();

		EasyMock.verify(this.matcher);
	}

	public void testVerifyTooFew() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.expect(this.matcher.getMethod()).andReturn(
				new MockMethod("foo", 3, null, true));
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		verifier.setInvocationCount(3);
		try {
			verifier.verify();
			fail();
		} catch (HammockException e) {
			assertEquals(
					IHammockExceptionErrors.METHOD_INVOKED_UNEXPECTED_NUMBER_OF_TIMES,
					e.getError());
		}

		EasyMock.verify(this.matcher);
	}

	public void testVerifyTooMany() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.expect(this.matcher.getMethod()).andReturn(
				new MockMethod("foo", 3, null, true));
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		verifier.setInvocationCount(1);
		try {
			verifier.verify();
			fail();
		} catch (HammockException e) {
			assertEquals(IHammockExceptionErrors.METHOD_INVOKED_TOO_MANY_TIMES,
					e.getError());
		}

		EasyMock.verify(this.matcher);
	}

	public void testGetInvocationCount() {
		EasyMock.expect(this.matcher.isMatch(mi1)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi2)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi3)).andReturn(false);
		EasyMock.expect(this.matcher.isMatch(mi4)).andReturn(true);
		EasyMock.expect(this.matcher.isMatch(mi5)).andReturn(true);
		EasyMock.replay(this.matcher);

		InvocationVerifier verifier = new InvocationVerifier(
				this.matcher, this.mi);
		assertEquals(2, verifier.getInvocationCount());
	}
}
